@@ -62,9 +62,17 @@ struct _CamelGroupwiseStorePrivate {
 	GHashTable *name_hash;/*get ids from names */
 	GHashTable *parent_hash;
 	EGwConnection *cnc;
+	CamelService *transport;
 };
+CamelServiceAuthType camel_groupwise_password_authtype = {
+	N_("Password"),
 
-extern CamelServiceAuthType camel_groupwise_password_authtype; /*for the query_auth_types function */
+	N_("This option will connect to the GroupWise server using a "
+	   "plaintext password."),
+
+	"",
+	TRUE
+};
 
 static GInitableIface *parent_initable_interface;
 
@@ -145,9 +153,6 @@ void
 groupwise_store_set_current_folder (CamelGroupwiseStore *groupwise_store,
                                     CamelFolder *folder)
 {
-
-	camel_service_lock (CAMEL_SERVICE (groupwise_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
-
 	if (groupwise_store->current_folder) {
 		g_object_unref (groupwise_store->current_folder);
 		groupwise_store->current_folder = NULL;
@@ -155,8 +160,6 @@ groupwise_store_set_current_folder (CamelGroupwiseStore *groupwise_store,
 
 	if (folder)
 		groupwise_store->current_folder = g_object_ref (folder);
-
-	camel_service_unlock (CAMEL_SERVICE (groupwise_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 }
 
 static gboolean
@@ -180,22 +183,16 @@ groupwise_connect_sync (CamelService *service,
 	if (status == CAMEL_SERVICE_DISCONNECTED)
 		return FALSE;
 
-	camel_service_lock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
-
-	if (priv->cnc) {
-		camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
+	if (priv->cnc)
 		return TRUE;
-	}
 
 	if (!check_for_connection (service, cancellable, error)) {
-		camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
-		camel_service_disconnect_sync (service, TRUE, NULL);
+		camel_service_disconnect_sync (service, TRUE, cancellable, NULL);
 		return FALSE;
 	}
 
 	if (!camel_session_authenticate_sync (session, service, NULL, cancellable, error)) {
-		camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
-		camel_service_disconnect_sync (service, TRUE, NULL);
+		camel_service_disconnect_sync (service, TRUE, cancellable, NULL);
 		return FALSE;
 	}
 
@@ -221,7 +218,6 @@ groupwise_connect_sync (CamelService *service,
 
 	camel_store_summary_save ((CamelStoreSummary *) store->summary);
 
-	camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 	if (E_IS_GW_CONNECTION (priv->cnc)) {
 		return TRUE;
 	}
@@ -229,6 +225,7 @@ groupwise_connect_sync (CamelService *service,
 	return FALSE;
 
 }
+
 #if 0
 static void
 groupwise_disconnect_cleanup (CamelService *service,
@@ -280,15 +277,14 @@ groupwise_disconnect_sync (CamelService *service,
 	CamelGroupwiseStore *groupwise_store = CAMEL_GROUPWISE_STORE (service);
 
 	if (clean) {
-		camel_service_lock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 		if (groupwise_store->priv && groupwise_store->priv->cnc) {
 			g_object_unref (groupwise_store->priv->cnc);
 			groupwise_store->priv->cnc = NULL;
+			if (groupwise_store->priv->transport)
+				camel_groupwise_transport_set_connection (groupwise_store->priv->transport, NULL);
 		}
 
 		groupwise_store_set_current_folder (groupwise_store, NULL);
-
-		camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 	}
 
 	/* groupwise_disconnect_cleanup (service, clean, ex); */
@@ -475,20 +471,14 @@ groupwise_store_get_folder_sync (CamelStore *store,
 	} else
 		g_clear_error (&local_error);
 
-	camel_service_lock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
-
 	groupwise_store_set_current_folder (gw_store, NULL);
 
-	if (!camel_groupwise_store_connected (gw_store, cancellable, error)) {
-		camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
+	if (!camel_groupwise_store_connected (gw_store, cancellable, error))
 		return NULL;
-	}
 
 	if (!E_IS_GW_CONNECTION ( priv->cnc)) {
-		if (!groupwise_connect_sync (service, cancellable, error)) {
-			camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
+		if (!groupwise_connect_sync (service, cancellable, error))
 			return NULL;
-		}
 	}
 
 	container_id =	g_strdup (g_hash_table_lookup (priv->name_hash, folder_name));
@@ -498,7 +488,6 @@ groupwise_store_get_folder_sync (CamelStore *store,
 	g_free (storage_path);
 	folder = camel_gw_folder_new (store, folder_name, folder_dir, cancellable, NULL);
 	if (!folder) {
-		camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 		g_set_error (
 			error, CAMEL_SERVICE_ERROR,
 			CAMEL_SERVICE_ERROR_INVALID,
@@ -532,7 +521,6 @@ groupwise_store_get_folder_sync (CamelStore *store,
 				&cursor);
 
 		if (status != E_GW_CONNECTION_STATUS_OK) {
-			camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 			g_free (container_id);
 			return NULL;
 		}
@@ -595,7 +583,6 @@ groupwise_store_get_folder_sync (CamelStore *store,
 	groupwise_store_set_current_folder (gw_store, folder);
 
 	g_free (container_id);
-	camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 
 	return folder;
 }
@@ -623,18 +610,12 @@ gw_store_reload_folder (CamelGroupwiseStore *gw_store,
 	name = camel_folder_get_display_name (folder);
 	full_name = camel_folder_get_full_name (folder);
 
-	camel_service_lock (CAMEL_SERVICE (gw_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
-
-	if (!camel_groupwise_store_connected (gw_store, cancellable, error)) {
-		camel_service_unlock (CAMEL_SERVICE (gw_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+	if (!camel_groupwise_store_connected (gw_store, cancellable, error))
 		return FALSE;
-	}
 
 	if (!E_IS_GW_CONNECTION ( priv->cnc)) {
-		if (!groupwise_connect_sync (CAMEL_SERVICE ((CamelStore *) gw_store), cancellable, error)) {
-			camel_service_unlock (CAMEL_SERVICE (gw_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+		if (!groupwise_connect_sync (CAMEL_SERVICE ((CamelStore *) gw_store), cancellable, error))
 			return FALSE;
-		}
 	}
 
 	container_id =	g_strdup (g_hash_table_lookup (priv->name_hash, full_name));
@@ -663,9 +644,8 @@ gw_store_reload_folder (CamelGroupwiseStore *gw_store,
 									NULL,
 									&cursor);
 			if (status != E_GW_CONNECTION_STATUS_OK) {
-					camel_service_unlock (CAMEL_SERVICE (gw_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
-					g_free (container_id);
-					return FALSE;
+				g_free (container_id);
+				return FALSE;
 			}
 
 			camel_operation_push_message (
@@ -678,7 +658,6 @@ gw_store_reload_folder (CamelGroupwiseStore *gw_store,
 									cursor, FALSE,
 									CURSOR_ITEM_LIMIT, position, &list);
 					if (status != E_GW_CONNECTION_STATUS_OK) {
-							camel_service_unlock (CAMEL_SERVICE (gw_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 							e_gw_connection_destroy_cursor (priv->cnc, container_id, cursor);
 							camel_folder_summary_save_to_db (folder->summary, NULL);
 							g_set_error (
@@ -733,7 +712,6 @@ gw_store_reload_folder (CamelGroupwiseStore *gw_store,
 	groupwise_store_set_current_folder (gw_store, NULL);
 
 	g_free (container_id);
-	camel_service_unlock (CAMEL_SERVICE (gw_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 	return TRUE;
 }
 
@@ -1005,21 +983,15 @@ groupwise_store_get_folder_info_sync (CamelStore *store,
 	/* Do not call groupwise_store_connected function as it would internall call folders_sync
 	 * to populate the hash table which is used for mapping container id */
 	if (!(camel_offline_store_get_online (CAMEL_OFFLINE_STORE (store))
-	    && camel_service_connect_sync ((CamelService *) store, error)))
+	    && camel_service_connect_sync ((CamelService *) store, cancellable, error)))
 		goto offline;
 
-	camel_service_lock (CAMEL_SERVICE (store), CAMEL_SERVICE_REC_CONNECT_LOCK);
-
-	if (!groupwise_folders_sync (groupwise_store, cancellable, error)) {
-		camel_service_unlock (CAMEL_SERVICE (store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+	if (!groupwise_folders_sync (groupwise_store, cancellable, error))
 		return NULL;
-	}
 
 	camel_store_summary_touch ((CamelStoreSummary *) groupwise_store->summary);
 	camel_store_summary_save ((CamelStoreSummary *) groupwise_store->summary);
 
-	camel_service_unlock (CAMEL_SERVICE (store), CAMEL_SERVICE_REC_CONNECT_LOCK);
-
 offline:
 	info = groupwise_get_folder_info_offline (store, top, flags, error);
 	return info;
@@ -1040,7 +1012,6 @@ create_junk_folder (CamelStore *store)
 	parent_id = "";
 	/* TODO: check for offlining*/
 
-	camel_service_lock (CAMEL_SERVICE (store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 	status = e_gw_connection_modify_junk_settings (priv->cnc, JUNK_ENABLE, 0, 0,  JUNK_PERSISTENCE);
 	if (status == E_GW_CONNECTION_STATUS_INVALID_CONNECTION)
 		status = e_gw_connection_modify_junk_settings (priv->cnc, JUNK_ENABLE, 0, 0,  JUNK_PERSISTENCE);
@@ -1057,7 +1028,6 @@ create_junk_folder (CamelStore *store)
 		g_hash_table_insert (priv->parent_hash, g_strdup (child_container_id), g_strdup (parent_id));
 		camel_store_folder_created (store, root);
 	}
-	camel_service_unlock (CAMEL_SERVICE (store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 
 	return root;
 }
@@ -1110,7 +1080,6 @@ groupwise_store_create_folder_sync (CamelStore *store,
 			return NULL;
 		}
 	}
-	camel_service_lock (CAMEL_SERVICE (store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 	status = e_gw_connection_create_folder (priv->cnc,parent_id,folder_name, &child_container_id);
 	if (status == E_GW_CONNECTION_STATUS_INVALID_CONNECTION)
 		status = e_gw_connection_create_folder (priv->cnc,parent_id,folder_name, &child_container_id);
@@ -1124,7 +1093,7 @@ groupwise_store_create_folder_sync (CamelStore *store,
 
 		camel_store_folder_created (store, root);
 	}
-	camel_service_unlock (CAMEL_SERVICE (store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+
 	return root;
 }
 
@@ -1139,12 +1108,8 @@ groupwise_store_delete_folder_sync (CamelStore *store,
 	EGwConnectionStatus status;
 	const gchar * container;
 
-	camel_service_lock (CAMEL_SERVICE (store), CAMEL_SERVICE_REC_CONNECT_LOCK);
-
-	if (!camel_groupwise_store_connected (groupwise_store, cancellable, error)) {
-		camel_service_unlock (CAMEL_SERVICE (store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+	if (!camel_groupwise_store_connected (groupwise_store, cancellable, error))
 		return FALSE;
-	}
 
 	container = g_hash_table_lookup (priv->name_hash, folder_name);
 
@@ -1162,7 +1127,6 @@ groupwise_store_delete_folder_sync (CamelStore *store,
 
 		g_hash_table_remove (priv->parent_hash, container);
 	}
-	camel_service_unlock (CAMEL_SERVICE (store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 
 	return TRUE;
 }
@@ -1193,12 +1157,8 @@ groupwise_store_rename_folder_sync (CamelStore *store,
 		return FALSE;
 	}
 
-	camel_service_lock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
-
-	if (!camel_groupwise_store_connected (groupwise_store, cancellable, error)) {
-		camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
+	if (!camel_groupwise_store_connected (groupwise_store, cancellable, error))
 		return FALSE;
-	}
 
 	groupwise_store_set_current_folder (groupwise_store, NULL);
 
@@ -1215,7 +1175,6 @@ groupwise_store_rename_folder_sync (CamelStore *store,
 			error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
 			_("Cannot rename GroupWise folder '%s' to '%s'"),
 			old_name, new_name);
-		camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 		return FALSE;
 	}
 
@@ -1238,7 +1197,6 @@ groupwise_store_rename_folder_sync (CamelStore *store,
 
 	g_free (oldpath);
 	g_free (newpath);
-	camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 
 	return TRUE;
 }
@@ -1349,7 +1307,7 @@ camel_groupwise_store_connected (CamelGroupwiseStore *store,
                                  GError **error)
 {
 	if (camel_offline_store_get_online (CAMEL_OFFLINE_STORE (store))
-	    && camel_service_connect_sync ((CamelService *) store, error)) {
+	    && camel_service_connect_sync ((CamelService *) store, cancellable, error)) {
 		CamelGroupwiseStore *gw_store = (CamelGroupwiseStore *) store;
 		CamelGroupwiseStorePrivate *priv = gw_store->priv;
 
@@ -1408,6 +1366,9 @@ groupwise_store_dispose (GObject *object)
 		groupwise_store->priv->cnc = NULL;
 	}
 
+	if (groupwise_store->priv->transport)
+		g_object_unref (groupwise_store->priv->transport);
+
 	/* Chain up to parent's dispose() method. */
 	G_OBJECT_CLASS (camel_groupwise_store_parent_class)->dispose (object);
 }
@@ -1484,6 +1445,8 @@ groupwise_store_authenticate_sync (CamelService *service,
 
 	priv->cnc = e_gw_connection_new_with_error_handler (
 		uri, user, password, &errors);
+	if (priv->transport)
+		camel_groupwise_transport_set_connection (priv->transport, priv->cnc);
 
 	g_free (uri);
 
@@ -1567,9 +1530,7 @@ groupwise_store_initable_init (GInitable *initable,
 			G_BINDING_BIDIRECTIONAL |
 			G_BINDING_SYNC_CREATE);
 
-		camel_groupwise_transport_set_store (
-			CAMEL_GROUPWISE_TRANSPORT (transport),
-			groupwise_store);
+		groupwise_store->priv->transport = g_object_ref (transport);
 	}
 
 	return (transport != NULL);
