@@ -167,7 +167,8 @@ message_info_from_db (CamelFolderSummary *s,
 		iinfo->server_flags = bdata_extract_digit (&part);
 	}
 
-	return info;}
+	return info;
+}
 
 static CamelMIRecord *
 message_info_to_db (CamelFolderSummary *s,
