@@ -31,7 +31,6 @@
 #include <glib/gi18n-lib.h>
 
 #include "camel-groupwise-settings.h"
-#include "camel-groupwise-store.h"
 #include "camel-groupwise-transport.h"
 #include "camel-groupwise-utils.h"
 
@@ -44,7 +43,7 @@
 G_DEFINE_TYPE (CamelGroupwiseTransport, camel_groupwise_transport, CAMEL_TYPE_TRANSPORT)
 
 struct _CamelGroupwiseTransportPrivate {
-	CamelGroupwiseStore *store;
+	EGwConnection *cnc;
 };
 
 static void
@@ -54,9 +53,9 @@ groupwise_transport_dispose (GObject *object)
 
 	priv = CAMEL_GROUPWISE_TRANSPORT_GET_PRIVATE (object);
 
-	if (priv->store != NULL) {
-		g_object_unref (priv->store);
-		priv->store = NULL;
+	if (priv->cnc) {
+		g_object_unref (priv->cnc);
+		priv->cnc = NULL;
 	}
 
 	/* Chain up to parent's dispose() method. */
@@ -109,7 +108,9 @@ groupwise_send_to_sync (CamelTransport *transport,
 	gchar *reply_request = NULL;
 	EGwItemLinkInfo *info = NULL;
 
-	if (!transport) {
+	priv = CAMEL_GROUPWISE_TRANSPORT_GET_PRIVATE (transport);
+
+	if (!priv->cnc) {
 		g_set_error (
 			error, CAMEL_SERVICE_ERROR,
 			CAMEL_SERVICE_ERROR_CANT_AUTHENTICATE,
@@ -117,21 +118,9 @@ groupwise_send_to_sync (CamelTransport *transport,
 		return FALSE;
 	}
 
-	priv = CAMEL_GROUPWISE_TRANSPORT_GET_PRIVATE (transport);
-
 	camel_operation_push_message (cancellable, _("Sending Message") );
 
-	/*camel groupwise store and cnc*/
-	cnc = cnc_lookup (priv->store->priv);
-	if (!cnc) {
-		g_warning ("||| Eh!!! Failure |||\n");
-		camel_operation_pop_message (cancellable);
-		g_set_error (
-			error, CAMEL_SERVICE_ERROR,
-			CAMEL_SERVICE_ERROR_CANT_AUTHENTICATE,
-			_("Authentication failed"));
-		return FALSE;
-	}
+	cnc = priv->cnc;
 
 	item = camel_groupwise_util_item_from_message (cnc, message, from);
 
@@ -215,11 +204,15 @@ camel_groupwise_transport_init (CamelGroupwiseTransport *transport)
 }
 
 void
-camel_groupwise_transport_set_store (CamelGroupwiseTransport *transport,
-                                     CamelGroupwiseStore *store)
+camel_groupwise_transport_set_connection (CamelService *service,
+                                          EGwConnection *cnc)
 {
-	g_return_if_fail (CAMEL_IS_GROUPWISE_TRANSPORT (transport));
-	g_return_if_fail (CAMEL_IS_GROUPWISE_STORE (store));
+	CamelGroupwiseTransport *transport = CAMEL_GROUPWISE_TRANSPORT (service);
 
-	transport->priv->store = g_object_ref (store);
+	g_return_if_fail (CAMEL_IS_GROUPWISE_TRANSPORT (transport));
+
+	if (transport->priv->cnc)
+		g_object_unref (transport->priv->cnc);
+	if (cnc)
+		transport->priv->cnc = g_object_ref (cnc);
 }
