@@ -26,6 +26,7 @@
 
 #include "camel-gw-listener.h"
 #include "camel-groupwise-settings.h"
+#include "camel-gwimap-settings.h"
 #include <string.h>
 #include <glib/gi18n-lib.h>
 #include <camel/camel.h>
@@ -133,12 +134,26 @@ static gboolean
 is_groupwise_account (EAccount *account)
 {
 	if (account->source->url != NULL) {
-		return (strncmp (account->source->url,  GROUPWISE_URI_PREFIX, GROUPWISE_PREFIX_LENGTH ) == 0);
+		return ((strncmp (account->source->url,  GROUPWISE_URI_PREFIX, GROUPWISE_PREFIX_LENGTH ) == 0) ||
+				(strncmp (account->source->url, "gwimap", 6 ) == 0));
 	} else {
 		return FALSE;
 	}
 }
 
+static CamelSettings *
+get_camel_settings_for_url (const gchar *source_url)
+{
+	if (strncmp (source_url, GROUPWISE_URI_PREFIX, GROUPWISE_PREFIX_LENGTH) == 0)
+		return g_object_new (CAMEL_TYPE_GROUPWISE_SETTINGS, NULL);
+	else if (strncmp (source_url, "gwimap", 6) == 0)
+		return g_object_new (CAMEL_TYPE_GWIMAP_SETTINGS, NULL);
+	else
+		g_assert_not_reached ();
+
+	return NULL;
+}
+
 /* looks up for an existing groupwise account info in the groupwise_accounts list based on uid */
 
 static GwAccountInfo *
@@ -201,7 +216,7 @@ add_esource (const gchar *conf_key,
 
 	group_name = info->name;
 
-	settings = g_object_new (CAMEL_TYPE_GROUPWISE_SETTINGS, NULL);
+	settings = get_camel_settings_for_url (info->source_url);
 	camel_settings_load_from_url (settings, url);
 
 	g_object_get (
@@ -409,7 +424,7 @@ modify_esource (const gchar *conf_key,
 
 	found_group = FALSE;
 
-	settings = g_object_new (CAMEL_TYPE_GROUPWISE_SETTINGS, NULL);
+	settings = get_camel_settings_for_url (old_account_info->source_url);
 	camel_settings_load_from_url (settings, new_url);
 
 	g_object_get (
@@ -562,7 +577,7 @@ get_addressbook_names_from_server (gchar *source_url)
 	if (!poa_address || strlen (poa_address) ==0)
 		return NULL;
 
-	settings = g_object_new (CAMEL_TYPE_GROUPWISE_SETTINGS, NULL);
+	settings = get_camel_settings_for_url (source_url);
 	camel_settings_load_from_url (settings, url);
 
 	g_object_get (
@@ -687,7 +702,7 @@ add_addressbook_sources (EAccount *account)
 	if (!poa_address || strlen (poa_address) ==0)
 		return FALSE;
 
-	settings = g_object_new (CAMEL_TYPE_GROUPWISE_SETTINGS, NULL);
+	settings = get_camel_settings_for_url (account->source->url);
 	camel_settings_load_from_url (settings, url);
 
 	g_object_get (
@@ -825,7 +840,7 @@ modify_addressbook_sources (EAccount *account,
 	if (!poa_address || strlen (poa_address) ==0)
 		return;
 
-	settings = g_object_new (CAMEL_TYPE_GROUPWISE_SETTINGS, NULL);
+	settings = get_camel_settings_for_url (account->source->url);
 	camel_settings_load_from_url (settings, url);
 
 	g_object_get (
@@ -1047,7 +1062,7 @@ account_changed (EAccountList *account_listener,
 		old_url = camel_url_new (existing_account_info->source_url, NULL);
 		old_poa_address = old_url->host;
 
-		settings = g_object_new (CAMEL_TYPE_GROUPWISE_SETTINGS, NULL);
+		settings = get_camel_settings_for_url (existing_account_info->source_url);
 		camel_settings_load_from_url (settings, old_url);
 
 		g_object_get (
@@ -1064,7 +1079,7 @@ account_changed (EAccountList *account_listener,
 		if (!new_poa_address || strlen (new_poa_address) ==0)
 			return;
 
-		settings = g_object_new (CAMEL_TYPE_GROUPWISE_SETTINGS, NULL);
+		settings = get_camel_settings_for_url (existing_account_info->source_url);
 		camel_settings_load_from_url (settings, new_url);
 
 		g_object_get (
