@@ -287,7 +287,7 @@ users_dialog_response (GtkWidget *dialog,
 	if (CAMEL_IS_VEE_STORE (store)) {
 		EFilterRule *rule;
 
-		rule = em_vfolder_rule_new (session);
+		rule = em_vfolder_editor_rule_new (session);
 		e_filter_rule_set_name (rule, folder_name);
 		vfolder_gui_add_rule (EM_VFOLDER_RULE (rule));
 		gtk_widget_destroy ((GtkWidget *) emfs);
